# -*- encoding: utf-8 -*-
require 'spec_helper'
#絵師

describe ArtistsController do
  before do
    Factory :admin
    @license = Factory :license
    @user = Factory( :user_yas)
    @author = @user.author
    @artist = Factory :artist_yas, :author_id => @author.id
  end
  
  describe '一覧表示に於いて' do
    before do
      Artist.stub(:list).and_return([@artist, @artist, @artist])
      sign_in @user
    end
    context '事前チェックする' do
      it '与えられたpageがセットされている' do
        get :index, :page => 5
        assigns(:page).should eq 5
      end
      it '省略されると@pageに1値が入る' do
        get :index
        assigns(:page).should eq 1
      end
      it '与えられたpage_sizeがセットされている' do
        get :index, :page_size => 15
        assigns(:page_size).should eq 15
      end
      it '省略されると@page_sizeにデフォルト値が入る' do
        get :index
        assigns(:page_size).should eq Artist.default_page_size
      end
      it '最大を超えると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 1500
        assigns(:page_size).should eq Artist.max_page_size
      end
      it '不正な値が入ると@page_sizeにデフォルト最大値が入る' do
        get :index, :page_size => 0
        assigns(:page_size).should eq Artist.default_page_size
      end
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :index
        response.should be_success 
      end
      it '絵師モデルに一覧を問い合わせている' do
        Artist.should_receive(:list).exactly(1)
        get :index
      end
      it '@artistsにリストを取得している' do
        get :index
        assigns(:artists).should have_at_least(3).items
      end
      context 'html形式' do
        it 'indexテンプレートを描画する' do
          get :index
          response.should render_template("index")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :index, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがリスト構造になっている' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.should have_at_least(3).items
        end
        it 'リストの先頭くらいは絵師っぽいものであって欲しい' do
          get :index, :format => :json
          json = JSON.parse response.body
          json.first.has_key?("name").should be_true
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :index
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :index
          response.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :index, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :index, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
  end
  
  describe '閲覧に於いて' do
    before do
      Artist.stub(:show).and_return(@artist)
      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :show, :id => @artist.id
        response.should be_success
      end
      it '絵師モデルに単体取得を問い合わせている' do
        Artist.should_receive(:show).exactly(1)
        get :show
      end
      it '@artistにアレを取得している' do
        get :show, :id => @artist.id
        assigns(:artist).should eq(@artist)
      end
      context 'html形式' do
        it 'showテンプレートを描画する' do
          get :show, :id => @artist.id
          response.should render_template("show")
        end
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :show, :id => @artist.id, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがアレになっている' do
          get :show, :id => @artist.id, :format => :json
          json = JSON.parse response.body
          json["name"].should match(/yas/)
        end
      end
    end
    context '作家権限がないとき' do
      before do
        sign_out @user
      end
      context 'html形式' do
        it 'ステータスコード302 Foundを返す' do
          get :show, :id => @artist.id
          response.status.should eq 302
        end
        it 'サインインページへ遷移する' do
          get :show, :id => @artist.id
          response.body.should redirect_to '/users/sign_in'
        end
      end
      context 'json形式' do
        it 'ステータスコード401 Unauthorizedを返す' do
          get :show, :id => @artist.id, :format => :json
          response.status.should eq 401
        end
        it '応答メッセージにUnauthorizedを返す' do
          get :show, :id => @artist.id, :format => :json
          response.message.should match(/Unauthorized/)
        end
      end
    end
=begin
    context '対象作家がないとき' do
      context 'html形式' do
        it '例外404 not_foundを返す' do
          lambda{
            get :show, :id => 0
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
      context 'json形式' do
        it '例外404 not_foundを返す' do
          lambda{ 
            get :show, :id => 0, :format => :json
          }.should raise_error(ActiveRecord::RecordNotFound)
        end
      end
    end
=end
  end
  
  describe '絵師数取得に於いて' do
    before do
      Artist.should_receive(:visible_count).and_return(3)
#      sign_in @user
    end
    context 'つつがなく終わるとき' do
      it 'ステータスコード200 OKを返す' do
        get :count, :format => :json
        response.should be_success 
      end
      context 'json形式' do
        it 'jsonデータを返す' do
          get :count, :format => :json
          lambda{JSON.parse(response.body)}.should_not raise_error(JSON::ParserError)
        end
        it 'データがHash構造になっていてコミック数が1である' do
          get :count, :format => :json
          json = JSON.parse response.body
          json["count"].should == 3
        end
      end
    end
  end

end
